#ifndef IR_GAMESTATE_H
#define IR_GAMESTATE_H

//------------------------------------------------------------------------------
/**
	GameState is displays a Graphical User Input aka "Main Menu"
*/

#include <time.h>
#include <fstream>
#include "OgreStableHeaders.h"
#include "GearState.h"
#include "GearBackgroundImage.h"
#include "IRRayCaster.h"
#include "IRSamplingType.h"
#include "IRLightSourceTriangle.h"

namespace IR {

class GameState : public Gear::State
{
public:
	GameState();
	virtual ~GameState(void);

	/// initialize State, load resources etc.
	void onStateEnter(Ogre::String prevState);
	/// end state and free resources
	void onStateLeave(Ogre::String nextState);

	/// called when attached to application object
	void onAttachToApplication();
	/// called when removed to application object
	void onRemoveFromApplication();

	/// called every frame, return next state
	Ogre::String onFrame();

	void toggleShadowDebugging(void);

	void instantRadiosityOnStateEnter(void);
	void instantRadiosityOnStateLeave(void);
	void instantRadiosityOnFrame(void);

private:
	IR::RayCaster*				m_RayCaster;
	Gear::BackgroundImage*		m_BackgroundImage;
	Ogre::Light**				m_IRLights;
	Ogre::Entity*				m_IRWandLamp;
	Ogre::ManualObject**		m_IRRays;
	Ogre::SceneNode**			m_IRLightNodes;
	Ogre::SceneNode**			m_IRRayNodes;
	Ogre::SceneNode*			m_IRRootNode;
	Ogre::SceneNode*			m_IRStaticSourceLightNode;
	Ogre::SceneNode*			m_IRMovableSourceLightNode;
	Ogre::Quaternion			m_IRSourceLightOrigOrientation; // I store here the quaternion to go from Blender coord system to Ogre's
	Ogre::Vector3				m_IRSourceLightOrigPosition;
	Ogre::Vector3				m_IROffsetFromWorldCoords;
	Ogre::MaterialPtr			m_IRRayMaterial;

	bool						m_bShadowDebugging;
	int							m_iOnFrameIterationCounter;

	bool						m_bRadiosity;
	bool						m_bPhotonDisplay;
	bool						m_bPrintLightValues;
	int							m_iNumSamples; // N in the paper
	int							m_iNumPhotons;
	int							m_iNumRays;
	Ogre::Real					m_fRho;

	IR::LightSourceTriangle		m_LightSourceTriangles[10];
	int							m_iNumLightSourceTriangles;

	SamplingType				m_eSamplingType;

	Ogre::Real					m_fCustomAttenationForRays;

	int							m_iRayDepthDisplayed;
	// -1	Show all lights
	//  0	Only show photons at light source, no rays
	//  1	Only show primary rays
	//  2	Only show secondary rays
	// ...
	//  8   Only show octonary rays
	//  9	Only show nonary rays

	bool						m_bUseWandForLight;
};

}; // end namespace
#endif