#include "IRLightSourceTriangle.h"

#include "Ogre.h"
#include "IRUtility.h"
#include "IRNumberGenerator.h"

namespace IR
{

void LightSourceTriangle::computeSamples(int numSamples, SamplingType type)
{
	this->numSamples = numSamples;

	std::vector<Ogre::Vector2> results;

	switch(type)
	{
	case UNIFORM:
		results = IR::UniformGenerator::getInstance().getPairs();
		break;
	case POISSON_DISK:
		IR::PoissonDiskGenerator::getInstance().setReflection(0);
		results = IR::PoissonDiskGenerator::getInstance().getPairs();
		break;
	case HALTON_SEQUENCE:
		results = IR::HaltonGenerator::getInstance().getPairs(numSamples, 1, 2);
		break;
	default:
		throw;
	}

	for (int sampleCount = 0; sampleCount < numSamples; sampleCount++)
	{
		double u = results[sampleCount].x;
		double v = results[sampleCount].y;

		double s = 1 - sqrt(1-u);
		double t = (1-s)*v;

		this->m_Samples[sampleCount] = m_Vertices[0] +
			s * (m_Vertices[1] - m_Vertices[0]) +
			t * (m_Vertices[2] - m_Vertices[0]);
	}
}

}